<?php

namespace Drupal\file_replace\Form;

use Drupal\Core\Entity\ContentEntityForm;
use Drupal\Core\Form\FormStateInterface;

/**
 * Form controller for the file replace forms.
 */
class FileReplaceForm extends ContentEntityForm {

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    /** @var \Drupal\file\FileInterface $file */
    $file = $this->getEntity();

    $form['original'] = [
      '#type' => 'fieldset',
      '#title' => t('Original'),
    ];
    $form['original']['link'] = [
      '#theme' => 'file_link',
      '#file' => $file,
      '#cache' => [
        'tags' => $file->getCacheTags(),
      ],
    ];

    $extension = pathinfo($file->getFileUri(), PATHINFO_EXTENSION);

    $form['replacement'] = [
      '#type' => 'fieldset',
      '#title' => t('Original'),
    ];
    $form['replacement']['replacement'] = [
      '#title' => t('Replacement'),
      '#type' => 'file',
      '#description' => $this->t('Select a file with extension .%extension and mimetype %mimetype to replace this file with.',
        [
          '%extension' => $extension,
          '%mimetype' => $file->getMimeType(),
        ]),
      '#upload_validators' => [
        'file_validate_extensions' => [$extension],
      ],
      '#attributes' => [
        'accept' => $file->getMimeType(),
      ],
    ];

    return parent::form($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    /** @var \Drupal\file\FileInterface $file */
    $file = $this->entity;
    $file_uri = $file->getFileUri();

    /** @var \Drupal\file\FileInterface $replacement */
    $replacement = file_save_upload('replacement', $form['replacement']['replacement']['#upload_validators'], FALSE, 0);
    if (!$replacement) {
      drupal_set_message(t('The replacement file was not saved'), 'error');
      return;
    }

    if (!file_move($replacement, $file_uri, FILE_EXISTS_REPLACE)) {
      drupal_set_message(t('The file could not be replaced'), 'error');
      return;
    }

    if (explode('/', $file->getMimeType())[0] == 'image') {
      /** @var \Drupal\image\Entity\ImageStyle[] $image_styles */
      $image_styles = $this->entityManager->getStorage('image_style')->loadMultiple();
      foreach ($image_styles as $image_style) {
        $image_style->flush($file_uri);
      }
    }

    drupal_set_message(t('The file was replaced.'));

    $file->save();
  }

}
